// ** EWM-900-FDTC Data Channel Encoding/Decoding Example
//
// REVISION:	A.0
// REV DATE:	8/23/01
// AUTHOR:	Steve Montgomery
// 
// COMPILER:	CCS PIC C Compiler V 2.726
// ********************************************

// ** Includes **


#include "\picc\include\16f877.h"

// ** Use **

#use delay(clock=4000000)
#use rs232(baud=19200, xmit=PIN_B1, rcv=PIN_B0)

// ** Define **

#define ACK 7

// ** Port Bit Definitions **

#byte port_b=6

#bit txd=port_b.1
#bit rxd=port_b.0

#byte port_c=7

#bit le=port_c.0
#bit dat=port_c.4
#bit clk=port_c.5

#bit lnaen=port_b.2


struct {
	int	opmode:2;
	int 	opchan:3;
	int	ver:3;
} dipsw;

#byte dipsw=8

// ** Mode Definitions **

int mode0, mode1, mode2;

#bit rxon_off=mode0.3
#bit txon_off=mode0.4

// ** Global Variables **
//

byte  encbyte[3];
byte  decbyte[3];
byte  enctable[16]={0x07,0x0b,0x0d,0x0e,0x13,0x15,0x16,0x19,0x1a,0x1c,0x23,0x25,0x26,0x29,0x2a,0x2c};
unsigned long 	timer_1;

// ** interrupt handlers **

#int_timer1

timer1_handler(){
	set_timer1(64535);
	timer_1++;
}



// ** load_refdiv **
//
// Loads the reference divider register

void load_refdiv()
{
	int   i;
	int   data;
	
	
	le=0;
	clk=0;
	dat=0;

	delay_ms(1);
	le=1;

   data=0;
   
   for(i=0;i<=7;i++) 
	{
		dat=data&0x80;
		delay_us(10);
		clk=1;
		delay_us(10);
		clk=0;	
		data<<=1;	
	}

   data=0x43;
   
   for(i=0;i<=7;i++) 
	{
		dat=data&0x80;
		delay_us(10);
		clk=1;
		delay_us(10);
		clk=0;	
		data<<=1;	
	}
	
   data=0xc0;
   
   for(i=0;i<=7;i++) 
	{
		dat=data&0x80;
		delay_us(10);
		clk=1;
		delay_us(10);
		clk=0;	
		data<<=1;	
	}

	// reset all serial lines to zero
	le=0;
	clk=0;
	dat=0;
}

// ** load_txchan **
//
// This function loads the m, a, and trim values into the txchan register

void load_txchan(unsigned long m_cnt, unsigned a_cnt, unsigned trim)
{
	int i;
	le=0;
	clk=0;
	dat=0;



	delay_ms(1);
	le=1;
	
	// send TRIM:0-2
	for(i=0;i<=2;i++)
	{
		dat=trim&0x4;
		delay_us(100);
		clk=1;
		delay_us(100);
		clk=0;
		trim<<=1;
	}


	// send out the lower 10 bits of m_cnt

	for(i=0;i<=9;i++)
	{
		dat=m_cnt&0x200;
		delay_us(100);
		clk=1;
		delay_us(100);
		clk=0;	
		m_cnt<<=1;	
	
	}
	// send out the lower 5 bits of a_cnt

	for(i=0;i<=4;i++)
	{
		dat=a_cnt&0x10;
		delay_us(100);
		clk=1;
		delay_us(100);
		clk=0;	
		a_cnt<<=1;	
	}
	
	// send f_cnt.. f_cnt is zero for this module
	for(i=0;i<=2;i++)
	{
		dat=0;
		delay_us(100);
		clk=1;
		delay_us(100);
		clk=0;
	}
	// clock in 0-1 on dat for the load control bits

	dat=1;
	delay_us(100);
	clk=1;
	delay_us(100);
	clk=0;
	dat=0;
	delay_us(100);
	clk=1;
	delay_us(100);
	clk=0;
	delay_us(100);
	delay_us(100);
	le=0;
	clk=0;
	dat=0;
}
// ** load_rxchan **
//
// This function loads the receive VCO register

void load_rxchan(unsigned long m_cnt, unsigned a_cnt, unsigned trim)
{
	int i;
	le=0;
	clk=0;
	dat=0;

	delay_ms(1);
	le=1;
	


	for(i=0;i<=1;i++)
	{
		dat=trim&0x2;
		delay_us(10);
		clk=1;
		delay_us(10);
		clk=0;
		trim<<=1;
	}

	
	// send out the lower 10 bits of m_cnt

	for(i=0;i<=9;i++)
	{
		dat=m_cnt&0x200;
		delay_us(10);
		clk=1;
		delay_us(10);
		clk=0;	
		m_cnt<<=1;	
	
	}
	// send out the lower 5 bits of a_cnt

	for(i=0;i<=4;i++)
	{
		dat=a_cnt&0x10;
		delay_us(10);
		clk=1;
		delay_us(10);
		clk=0;	
		a_cnt<<=1;	
	}
	
	// send f_cnt... f_cnt=0
	for(i=0;i<=2;i++)
	{
		dat=0;
		delay_us(10);
		clk=1;
		delay_us(10);
		clk=0;
	}
	// clock in 1-0 on dat for the load control bits

	dat=0;
	delay_us(10);
	clk=1;
	delay_us(10);
	clk=0;
	dat=1;
	delay_us(10);
	clk=1;
	delay_us(10);
	clk=0;
	delay_us(10);
	le=0;
	clk=0;
	dat=0;
}

// ** init_tcmode **
//
// Resets the mode register to 0.

void init_tcmode(void)
{
   mode0=0x27;
   mode1=0;
   mode2=0;
}

// ** load_tcmode **
//
// Loads the mode register

void load_tcmode(void)
{
	int i;	// index variable


	le=0;
	clk=0;
	dat=0;
	delay_ms(1);
	
	le=1;

	for(i=0;i<=7;i++)
	{
		dat=mode2&0x80;
		delay_us(10);
		clk=1;
		delay_us(10);
		clk=0;	
		mode2<<=1;	
	}
	for(i=0;i<=7;i++)
	{
		dat=mode1&0x80;
		delay_us(10);
		clk=1;
		delay_us(10);
		clk=0;	
		mode1<<=1;	
	}
	for(i=0;i<=7;i++)
	{
		dat=mode0&0x80;
		delay_us(10);
		clk=1;
		delay_us(10);
		clk=0;	
		mode0<<=1;	
	}
	le=0;
	clk=0;
	dat=0;

	load_refdiv();
}

void encode(byte b0, byte b1)
{
   int   index;

   // encode nibble 0

   encbyte[0]=enctable[b0&0x0f];
   
   // encode nibble 1
   
   encbyte[0]+=(enctable[b0&0xf0]<<6);
   encbyte[1]=(enctable[b0&0xf0]>>2);
   
   // encode nibble 2
   
   encbyte[1]+=(enctable[b1&0x0f]<<4);
   encbyte[2]=(enctable[b1&0x0f]>>4);
   
   // encode nibble 3
   
   encbyte[2]+=(enctable[b1&0xf0]<<2);
   

}
byte dectrans(byte index)
{
   byte i;
   
   for(i=0;i<=15;i++)
   {
      if(enctable[i]==index)
         break;
   }
   return i;
}
void decoder(byte b0, byte b1, byte b2)
{
   byte index;
   
   // decode nibble 0
   
   index=(b0&0x3f);
   decbyte[0]=dectrans(index);
   
   // decode nibble 1
   
   index=(b0&0xc0)>>6;
   index+=(b1&0x0f)<<2;
   decbyte[0]+=dectrans(index);
   
   // decode nibble 2
   
   index=(b1&0xf0)>>4;
   index+=(b2&0x03)<<4;
   decbyte[1]=dectrans(index);
   
   // decode nibble 3
   
   index=(b2&0xfc)>>2;
   decbyte[1]+=dectrans(index);
   
}
void putc_coded(b0, b1)
{
   encode(b0,b1);
   putc(encbyte[0]);
   putc(encbyte[1]);
   putc(encbyte[2]);
}
void init_example()
{
	timer_1=0;
	setup_timer_1(T1_INTERNAL | T1_DIV_BY_1);
	setup_counters(RTCC_INTERNAL, RTCC_DIV_2);
	enable_interrupts(int_timer1);
	enable_interrupts(global);	
	lnaen=0;
	set_tris_b(0xf9);	// set-up port b port directions
	set_tris_c(0x8a);	// set-up port c port directions
	
	
	init_tcmode();
	load_tcmode();		// make sure that everything is off

	setup_adc(ALL_ANALOG);
	set_adc_channel(0);
	init_tcmode();
	disable_interrupts(global);
	switch(dipsw.opmode)
	{
		case 0:  txon_off=1; load_tcmode(); load_txchan(564,23,3); break;
		case 1:  rxon_off=1; lnaen=1; load_tcmode(); load_rxchan(564,23,3); break;
	}

}
void main(void)
{
	int a;
	byte inbyte[6];
	
   init_example();
   



   if(dipsw.opmode==0)
      while(1)
      {
         delay_ms(500); // two packets per second
         
         /* send the pre-amble */
         putc(0x55);
         putc(0x55);
         
         /* send the uart sync byte */
         putc(0xff);
         
         /* send the start code */
         putc(0x00);
         putc(0xfe);
         
         /* send the data */
         putc_coded(0,1);
         putc_coded(2,3);
      }
   else
      while(1)
      {
         while(getch()!=0x00);   // wait for the start byte
         if(getch()==0xfe)
         {
            for(a=0;a<=5;a++)
               inbyte[a]=getch();
            decoder(inbyte[0],inbyte[1],inbyte[2]);
            putc(decbyte[0]);
            putc(decbyte[1]);
            decoder(inbyte[3],inbyte[4],inbyte[5]);
            putc(decbyte[0]);
            putc(decbyte[1]);
         }
      }

}
